function [smoothedLabels,bf,org_l] = RaguSmoothLabels(V,M,b,PenaltyPar,start,stop)
% function [l,org_l] = RaguSmoothLabels(V,M,b,lambda)
%
% Where  V is the voltage vector(Ne x Nt)
%        M is the microstate vector(Nu x Ne)
%        b is the window size
%       lambda is the non-smoothness penalty factor (should be between 0 and 1)
%

% Copyright 2009-2011 Thomas Koenig
% distributed under the terms of the GNU AFFERO General Public License


if nargin < 5
    start = 1;
end

if nargin < 6
    stop = size(V,2);
end

[Ns,Nt] = size(V);
Nu = size(M,1);

fit = M * V;

if(Nu == 1)
    smoothedLabels = ones(1,Nt);
    bf = fit;
    org_l = smoothedLabels;
    return
end

% Line 1

s0 = 0;
ConvergencePar = 1e-6;

su = 1;


% Line 2
[msFit,L] = max(fit,[],1);

% Line 3
Lambda = L;

% Line 4
Vvar = sum(V.*V,1);
%e = (sum(Vvar) - mx'*mx) / (Nt * (Ns - 1)); removed 22.1.2020
e = sum(Vvar - msFit.*msFit) / (Nt * (Ns - 1)); %added 22.1.2020


% Helper stuff

HitMatrix = repmat((1:Nu)',1,Nt);
bWindow = ones(1,2*b + 1);
ClassResiduals = (repmat(Vvar,Nu,1) - fit);

if nargout > 3
    org_l = zeros(Nt,1);
    org_l(start:stop) = L(1,start:stop)';
end

while true
    % Line 5a
    Hits = repmat(L,Nu,1) == HitMatrix;
    n_kt = conv2(Hits,bWindow,'same');
    
    %Line 5b
    penalizedFit  = ClassResiduals  / (2 * e * (Ns-1)) - PenaltyPar * n_kt;
    [~,Lambda] = min(penalizedFit,[],1);
    
    % Line 6 
    L = Lambda;
    for t = 1:Nt
        msFit(1,t) = M(L(1,t),:) * V(:,t);
    end
    
    % Line 7
    su = sum(Vvar-msFit.*msFit,2) / (Nt * (Ns-1));
    
    % Line 8
    if (abs(s0-su) <= (su * ConvergencePar))
        break;
    else
        s0 = su;
    end
end

smoothedLabels = L;

if nargout > 1
    bf = msFit';
end

